<?php

/**
 * Operations for players
 *
 * This class handles all data related to the current player.
 *
 */
class Player {

  /**
   * Get the players current location
   *
   * This function will return the record id of the sector that the player is
   * currently located at, it does that using the username of the player and then
   * getting his/her player id to finally get the player location.
   *
   * @param string $username
   * @return integer Record id of the current sector
   */
  function getplayerloc($username,$gameid) {
    $playerid = player::getplayerid($username,$gameid);

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $sec = $ra['location'];
    }

    return $sec;
  }

  /**
   * Get the player id from the username
   *
   * This function will return the player id of the currently logged in username.
   *
   * @param string $username
   * @return integer The player id of the user
   */
  function getplayerid($username,$gameid) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    if (dbaccess::numresults($resc) > 0) {
      while ($ra = dbaccess::getarray($resc)) {
        $userid = $ra['id'];
      }
      $resa = dbaccess::query("SELECT id FROM `".$gameid."_players` WHERE userid='$userid'");
      $resa = dbaccess::getrow($resa);
      $playerid = $resa[0];
    } else {
      $playerid = 0;
    }

    return $playerid;
  }

  /**
   * Get the player id using the player name
   *
   * This function will accept the player name, and then return his/her player id
   *
   * @param string $playername
   * @return integer The player id of the player
   */
  function getplayeridfpn($playername,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE playername='$playername'");
    if (dbaccess::numresults($resc) > 0) {
      while ($ra = dbaccess::getarray($resc)) {
        $playerid = $ra['id'];
      }
    } else {
      $playerid = 0;
    }

    return $playerid;
  }

  /**
   * Get the user id of the username
   *
   * This function will return the user id of the passed username.
   *
   * @param string $username
   * @return integer The user id of the username
   */
  function getuserid($username) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    while ($ra = dbaccess::getarray($resc)) {
      $ids = $ra['id'];
    }

    return $ids;

  }

  /**
   * Get the time zone for a user
   *
   * This function will accept a username and then return the timezone
   *
   * @param string $username
   * @return integer The timezone for the user
   */
  function getTimeZone($username) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    while ($ra = dbaccess::getarray($resc)) {
      $tzone = $ra['toffset'];
    }

    return $tzone;
  }

  /**
   * Return the id of the port where the player is docked
   *
   * This function will accept the players id, and then return the port id of
   * where the player is currently docked, if the player is not docked at the
   * moment this function will return 0.
   *
   * @param integer $playerid
   * @return integer The id of the port
   */
  function hasdocked($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $dockedat = $ra['docked'];
    }

    return $dockedat;
  }

  /**
   * Return the id of the space station where the player is docked
   *
   * This function will accept the players id, and then return the space station id of
   * where the player is currently docked, if the player is not docked at the
   * moment this function will return 0.
   *
   * @param integer $playerid
   * @return integer The id of the space station
   */
  function hasdockedstation($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $dockedat = $ra['atstation'];
    }

    return $dockedat;
  }

  /**
   * Get all the information on a player
   *
   * This function will accept the player id of a player and then return everything
   * about that player, which is current credits, hull etc.
   *
   * @param integer $playerid
   * @return array All the information about a player
   */
  function getplayerinfo($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $playerinfo['name']       = stripslashes($ra['playername']);
      $playerinfo['auth']       = $ra['auth'];
      $playerinfo['ship']       = stripslashes($ra['shipname']);
      $playerinfo['alliance']   = $ra['allianceid'];
      $playerinfo['credits']    = $ra['credits'];
      $playerinfo['turns']      = $ra['turns'];
      $playerinfo['newbie']     = $ra['newbie'];
      $playerinfo['cargo']      = $ra['cargo'];
      $playerinfo['shipid']     = $ra['shipid'];
      $playerinfo['location']   = $ra['location'];
      $playerinfo['exp']        = $ra['exp'];
      $playerinfo['docked']     = $ra['docked'];
      $playerinfo['userid']     = $ra['userid'];
      $playerinfo['banner']     = $ra['playerban'];
      $playerinfo['galid']      = $ra['galid'];
      $playerinfo['statid']     = $ra['atstation'];
    }

    $old_exp = 0;
    $init_count = 5;
    $lvl_counter = 1;
    $gotlevel = FALSE;

    while (!$gotlevel) {
      if (($playerinfo['exp'] >= $old_exp) and ($playerinfo['exp'] <= $init_count)) {
        $gotlevel = TRUE;
      } else {
        $lvl_counter += 1;
        $old_exp = $init_count;
        $init_count = $init_count + (103 * (pow($lvl_counter,2.1)));
      }
    }

    $playerinfo['level'] = $lvl_counter - 1;

    $playerinfo['req_exp'] = $init_count;

    return $playerinfo;
  }

  /**
   * Decrease a players credits
   *
   * Accepts the players id and the amount of credits to decrease by.
   *
   * @param integer $playerid
   * @param integer $creds
   */
  function downcreds($playerid, $creds, $gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentcreds = $playerinf['credits'];
    $newcreds = $currentcreds - $creds;

    dbaccess::query("UPDATE `".$gameid."_players` SET credits = '$newcreds' WHERE id='$playerid'");
  }

  /**
   * Increase a players credits
   *
   * Accepts the players id and the amount of credits to increase by
   *
   * @param integer $playerid
   * @param integer $creds
   */
  function upcreds($playerid, $creds, $gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentcreds = $playerinf['credits'];
    $newcreds = $currentcreds + $creds;

    dbaccess::query("UPDATE `".$gameid."_players` SET credits = '$newcreds' WHERE id='$playerid'");
  }

  /**
   * Increase a players experience
   *
   * Accepts the players id and the amount of experience to increase by
   *
   * @param integer $playerid
   * @param integer $exp
   */
  function upexp($playerid, $exp,$gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentexp = $playerinf['exp'];
    $newexp = $currentexp + $exp;

    dbaccess::query("UPDATE `".$gameid."_players` SET exp = '$newexp' WHERE id='$playerid'");
  }

  /**
   * Decrease a players experience
   *
   * Accepts the players id and the amount of experience to decrease by
   *
   * @param integer $playerid
   * @param integer $exp
   */
  function downexp($playerid, $exp, $gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentexp = $playerinf['exp'];
    $newexp = $currentexp - $exp;

    dbaccess::query("UPDATE `".$gameid."_players` SET exp = '$newexp' WHERE id='$playerid'");
  }

  /**
   * Increase a players amount of turns
   *
   * This function will increase the amount of turns a player has, it is mainly
   * used by the event processor
   *
   * @param integer $playerid
   * @param integer $amnt
   */
  function upturns($playerid, $amnt, $gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentturns = $playerinf['turns'];
    $newturns = $currentturns + $amnt;
    dbaccess::query("UPDATE `".$gameid."_players` SET turns = '$newturns' WHERE id='$playerid'");
  }

  /**
   * Decrease a players amount of turns
   *
   * This function will decrease the amount of turns a player has.
   *
   * @param integer $playerid
   * @param integer $amnt
   */
  function downturns($playerid, $amnt, $gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $currentturns = $playerinf['turns'];
    $newturns = $currentturns - $amnt;
    dbaccess::query("UPDATE `".$gameid."_players` SET turns = '$newturns' WHERE id='$playerid'");
  }

  /**
   * Log a person in
   *
   * Log a person in
   *
   * @param string $username
   */
  function login($username) {
    dbaccess::query("UPDATE users SET loggedin='1' WHERE username='$username'");
  }

  /**
   * Log a person out
   *
   * Log a person out
   *
   * @param string $username
   */
  function logout($username) {
    dbaccess::query("UPDATE users SET loggedin='0' WHERE username='$username'");
  }

  /**
   * Check login validity
   *
   * This function will check the session variables according to what is in the
   * database, if something is wrong, it will return false and the player will
   * be logged out.
   *
   * {@source}
   *
   * @return boolean Valid or not.
   */
  function checklogin() {
    global $users;
    if (!empty($_SESSION['username'])) {
      $username = $_SESSION['username'];
      $password = $_SESSION['password'];

      $resc = dbaccess::query("SELECT * FROM users
                               WHERE username='$username'");

      while ($ra = dbaccess::getarray($resc)) {
        $user_db = $ra['username'];
        $pass_db = $ra['password'];
        $lastactive_db = $ra['lastactive'];
      }

      if ($password == $pass_db) {
        $lastactive = time();
        if (($lastactive - $users['logout_time']) >= $lastactive_db) {
          // Person was inactive for too long
          player::logout($username);
          dbaccess::query("UPDATE users SET lastactive='$lastactive'
                           WHERE username='$username'");
          return FALSE;
        } else {
          dbaccess::query("UPDATE users SET lastactive='$lastactive'
                           WHERE username='$username'");
          return TRUE;
        }
      } else {
        return FALSE;
        player::logout($username);
      }
    } else {
      return FALSE;
    }
  }

  /**
   * Return how many new messages a player has
   *
   * This function will check a players inbox and return how many messages are
   * new.
   *
   * @param integer $playerid
   * @return integer New mail
   */
  function getnewmail($playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_messages` WHERE `to` = '$playerid' AND `read` = '0'");
    $new = dbaccess::numresults($res);

    return $new;
  }

  /**
   * Return username of playerid
   *
   * This function will return the username of a player id.
   *
   * @param integer $playerid
   * @return string The username
   */
  function getusername($playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($res)) {
      $uid = $ra['userid'];
    }

    $res = dbaccess::query("SELECT * FROM users WHERE id='$uid'");
    while ($ra = dbaccess::getarray($res)) {
      $username = $ra['username'];
    }

    return $username;
  }

  function playerhasdrones($sec,$playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_drones` WHERE secid='$sec' AND owner='$playerid'");
    $hasds = dbaccess::numresults($res);

    if ($hasds > 0) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

}

?>
