<?php

/**
 * Functions concerning a port
 *
 * This class will handle all information required about the current port.
 *
 */
class Port {

  /**
   * Return port name
   *
   * This function will return the name of the current port, if the port is
   * a station it will return a name, otherwise it will return the sector id
   * (the one displayed in the game).
   *
   * @param integer $id
   * @return mixed Name or Id of port
   */
  function getportname($id,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$id'");
    $hasport = dbaccess::numresults($resc);
    while ($rc = dbaccess::getArray($resc)) {
      $port_name = $rc['port_name'];
      $sector_id = $rc['sector_id'];
    }

    if ($hasport > 0) {
      if (empty($port_name)) {
        return $sector_id;
      } else {
        return $port_name;
      }
    } else {
      return FALSE;
    }
  }

  /**
   * Get the selling price of a upgrade good
   *
   * This function will calculate the selling price of a good depending on how much
   * stock is at that location, hence the dynamic prices.
   *
   * @param integer $good
   * @param integer $qty
   * @return integer The selling price of a good
   */
  function getupprice($good,$qty) {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$good'");
    while ($rc = dbaccess::getArray($resc)) {
      $goodvalue = $rc['value'];
    }

    $max_demand = 30000;
    //$qty = $max_demand - $qty;

    //$qty = 1/(pow(($qty),0.5));
    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $value = pow(($qty),0.35)*$goodvalue;
    //$value = sqrt((1/pow(($qty/$max_demand),0.5))*sqrt(($max_demand - $qty))*$goodvalue);

	//settype($value,"integer");

	return $value;
  }

  /**
   * Get the selling price of a good
   *
   * This function will calculate the selling price of a good depending on how much
   * stock is at that location, hence the dynamic prices.
   *
   * @param integer $good
   * @param integer $qty
   * @return integer The selling price of a good
   */
  function getsellprice($good,$qty,$up = '0') {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$good'");
    while ($rc = dbaccess::getArray($resc)) {
      $goodvalue = $rc['value'];
    }

    if ($up == 1) {
        $max_demand = 30000;
    } else {
        $max_demand = 10000;
    }

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $value = pow(($max_demand/$qty),0.8)*$goodvalue;

	settype($value,"integer");

	return $value;
  }

  /**
   * Get the buying price of a good
   *
   * This function will calculate the buying price of a good depending on how
   * much stock is available.
   *
   * @param integer $good
   * @param integer $qty
   * @return integer The buying price of a good
   */
  function getbuyprice($good,$qty) {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$good'");
    while ($rc = dbaccess::getArray($resc)) {
      $goodvalue = $rc['value'];
    }

    $max_demand = 10000;

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $value = sqrt((1/pow(($qty/$max_demand),0.5))*sqrt(($max_demand - $qty))*$goodvalue);

	settype($value,"integer");

	return $value;
  }

  /**
   * Get the experience gained from a purchase
   *
   * This function will calculate the experience that is gained from a purchase
   * from a port.
   *
   * @param integer $good
   * @param integer $qty
   * @param integer $amnt
   * @return integer The experience gained from the purchase
   */
  function getexpbuy($good,$qty,$amnt) {

    $max_demand = 10000;

    $sellp = $this->getsellprice($good,$qty);
    $experience = (pow($max_demand/$qty,0.7) * $amnt) / ($sellp/3.5);

	settype($experience,"integer");

	if ($experience <0)$experience=0;

	return $experience;
  }

  /**
   * Get the experience gained from a sale
   *
   * This function will calculate the experience that is gained from a sale to
   * a port.
   *
   * @param integer $good
   * @param integer $qty
   * @param integer $amnt
   * @return integer The experience gained from the sale
   */
  function getexpsell($good,$qty,$amnt,$up = '0') {
    $max_demand = 10000;

    $buyp = $this->getbuyprice($good,$qty);

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $experience = sqrt((pow(1/($qty/$max_demand),0.7) * $amnt) * ($buyp));

	settype($experience,"integer");

	if ($experience <0)$experience=0;

	return $experience;
  }

  /**
   * Return the id of the port where the player is docked
   *
   * This function will accept the players id, and then return the port id of
   * where the player is currently docked, if the player is not docked at the
   * moment this function will return 0.
   *
   * @param integer $playerid
   * @return integer The id of the port
   */
  function hasdocked($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $dockedat = $ra['docked'];
    }

    return $dockedat;
  }

  /**
   * Dock a player
   *
   * This function will accept the player id, and port id of where the player
   * wishes to dock, and then it will dock the player there.
   *
   * @param integer $playerid
   * @param integer $portid
   */
  function dock($playerid, $portid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET docked='$portid' WHERE id='$playerid'");
  }

  /**
   * Undock a player
   *
   * This function will accept the player id of a player and then set their current
   * docked port to 0, which then makes all the other functions return that the
   * player is not longer docked.
   *
   * @param integer $playerid
   */
  function undock($playerid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET docked='0' WHERE id='$playerid'");
  }

  /**
   * Generate new upgrade goods
   *
   * Up for revamping
   *
   * @todo Redo this function for the new system
   */
  function newupgoods($portid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$portid'");
    while ($r = dbaccess::getarray($res)) {
      $sells = $r['sells'];
      $buys  = $r['buys'];
      $level = $r['level'];
    }

    unset($newupgoods);
    unset($newupreqs);
    $newupgoods = array();
    $newupreqs = array();

    $res = dbaccess::query("SELECT * FROM goods WHERE level <= '$level'");
    while ($r = dbaccess::getarray($res)) {
      $newupgoods[] = $r['id'];
      $newupreqs[] = $r['id'];
    }

    $sells = explode(",", $sells);
    foreach ($sells as $i => $value) {
      $arrkey = array_search($value, $newupgoods);
      unset($newupgoods[$arrkey]);
    }

    $buys = explode(",", $buys);
    foreach ($buys as $i => $value) {
      $arrkey = array_search($value, $newupgoods);
      unset($newupgoods[$arrkey]);
    }

    if (empty($newupgoods)) {
      $nlvl = $level + 1;
      dbaccess::query("UPDATE `".$gameid."_ports` SET level='$nlvl' WHERE id='$portid'");
      return FALSE;
    } else {
      $thenewgood = array_rand($newupgoods);
      unset($newupreqs[$thenewgood]);
      $thenewgood = $newupgoods[$thenewgood];

      foreach($sells as $i => $value) {
        $arrkey = array_search($value, $newupreqs);
        unset($newupreqs[$arrkey]);
      }

      $arrkey = array_search($thenewgood, $newupreqs);
      unset($newupreqs[$arrkey]);

      $newstring_ups = $thenewgood.":";

      $thenewreqs = array_rand($newupreqs,2);

      unset($fordbreqs);
      $fordbreqs = array();
      unset($fordbqty);
      $fordbqty = array();

      foreach ($thenewreqs as $i => $value) {
        $fordbreqs[] = $value;
        $fordbqty[] = $value."+30000";
      }

      $fordbreqs = implode("-", $fordbreqs);
      $fordbqty = implode("@", $fordbqty);

      $newstring_ups = $newstring_ups.$fordbreqs;
      $newstring_qty = $thenewgood.':'.$fordbqty;

      dbaccess::query("UPDATE `".$gameid."_ports` SET upgrades = '$newstring_ups',
                                        upgradeqty = '$newstring_qty'
                       WHERE id='$portid'");

      return TRUE;
    }
  }

}

?>
