<?php

/**
 * Ship class
 *
 * Every function concerning a ship is stored here.
 */
class Ship {

  /**
   * Return info on a ship
   *
   * This function will return everything about a ship, it also calculates a lot of
   * other information that is useful to the game. It accepts the player id, the
   * hull id parameter is optional, the Ship Dealer script uses it to get information
   * on the hulls that the dealer sells.
   *
   * @param integer $playerid
   * @param integer $hullid
   * @return array Info on the ship or hull
   */

  function getshipinfo($playerid,$gameid,$hullid = '') {
    $technology = tech::gettechinfo();
    $playerinfo = player::getplayerinfo($playerid,$gameid);
    $shipid = $playerinfo['shipid'];

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ships` WHERE id='$shipid'");
    while ($ra = dbaccess::getarray($resc)) {
      $hull = $ra['hullid'];
      $properties = $ra['properties'];
      $equipment = $ra['equipment'];
      $weapons = $ra['weapons'];
    }

    if ($weapons == " ") {
      unset($weapons);
    }

    $shipinfo['hullid']    = $hull;
    $shipinfo['weapons']   = $weapons;
    $shipinfo['equipment'] = $equipment;

    if (!empty($hullid)) {
      $hull = $hullid;
    }

    $resc = dbaccess::query("SELECT * FROM hulls WHERE id='$hull'");
    while ($ra = dbaccess::getarray($resc)) {
      $shipinfo['hullname']    = $ra['name'];
      $shipinfo['cargoslots']  = $ra['cargoslots'];
      $shipinfo['shieldslots'] = $ra['shieldslots'];
      $shipinfo['armourslots'] = $ra['armourslots'];
      $shipinfo['engineslots'] = $ra['engineslots'];
      $shipinfo['equipslots']  = $ra['equipslots'];
      $shipinfo['primslots']   = $ra['primslots'];
      $shipinfo['secslots']    = $ra['secslots'];
      $shipinfo['tps']         = $ra['tps'];
      $shipinfo['startprop']   = $ra['startprop'];
    }

    if (!empty($equipment)) {
      $tequip = explode(",", $equipment);
      $key = array_search("bck", $tequip);

      if ($key !== FALSE) {
        $shipinfo['tps'] += 10;
      }
    }

    $props = explode(",", $properties);
    foreach ($props as $i => $value) {
      $cprops = explode(":", $value);
      $cshipprops[$cprops[0]] = $cprops[1];
    }

    $propshull = explode(",", $shipinfo['startprop']);
    foreach ($propshull as $i => $value) {
      $cpropshull = explode(":", $value);
      $cshippropshull[$cpropshull[0]] = $cpropshull[1];
    }

    $ship_tech['shields'] = $technology[1]['prop'];
    $ship_tech['armour']  = $technology[2]['prop'];
    $ship_tech['cargo']   = $technology[3]['prop'];
    $ship_tech['engines'] = $technology[4]['prop'];

    $max_shields_h = explode(":", $ship_tech['shields']);
    $max_shields = $max_shields_h[1];
    $max_shields *= $shipinfo['shieldslots'];

    $max_armour_h = explode(":", $ship_tech['armour']);
    $max_armour = $max_armour_h[1];
    $max_armour *= $shipinfo['armourslots'];

    $max_holds_h = explode(":", $ship_tech['cargo']);
    $max_holds = $max_holds_h[1];
    $max_holds *= $shipinfo['cargoslots'];

    $max_engines_h = explode(":", $ship_tech['engines']);
    $max_engines = $max_engines_h[1];
    $max_engines *= $shipinfo['engineslots'];

    $shipinfo['shipid']         = $shipid;
    $shipinfo['currentshields'] = $cshipprops['S'];
    $shipinfo['currentarmour']  = $cshipprops['A'];
    $shipinfo['currentholds']   = $cshipprops['C'];
    $shipinfo['currentengines'] = $cshipprops['E'];
    $shipinfo['hullshields']    = $cshippropshull['S'];
    $shipinfo['hullarmour']     = $cshippropshull['A'];
    $shipinfo['hullholds']      = $cshippropshull['C'];
    $shipinfo['hullengines']    = $cshippropshull['E'];
    $shipinfo['maxshields']     = $max_shields;
    $shipinfo['maxarmour']      = $max_armour;
    $shipinfo['maxholds']       = $max_holds;
    $shipinfo['maxengines']     = $max_engines;

    $shipinfo['currentcd']      = 0;
    $shipinfo['currentsd']      = 0;

    $currentdrones = 0;
    if (!empty($cshipprops['CD'])) {
      $currentdrones += $cshipprops['CD'];
      $shipinfo['currentcd'] = $cshipprops['CD'];
    }

    if (!empty($cshipprops['SD'])) {
      $currentdrones += $cshipprops['SD'];
      $shipinfo['currentsd'] = $cshipprops['SD'];
    }

    $shipinfo['currentdrones'] = $currentdrones;

    $lvl_tmp = 0;
    $lvl_tmp += ($shipinfo['currentshields'] / $max_shields_h[1]);
    $lvl_tmp += ($shipinfo['currentarmour'] / $max_armour_h[1]);
    $lvl_tmp += ($shipinfo['currentholds'] / $max_holds_h[1]);
    $lvl_tmp += ($shipinfo['currentengines'] / $max_engines_h[1]);

    if (empty($weapons)) {
      $weapons = array();
    } else {
      $weapons = explode(",", $weapons);
    }
    $lvl_tmp += array_sum($weapons);

    $shipinfo['level'] = ceil($lvl_tmp/15);

    return $shipinfo;
  }

  function getshipinfo_parked($shipid,$gameid,$hullid = '') {
    $technology = tech::gettechinfo();

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ships` WHERE id='$shipid'");
    while ($ra = dbaccess::getarray($resc)) {
      $hull = $ra['hullid'];
      $properties = $ra['properties'];
      $equipment = $ra['equipment'];
      $weapons = $ra['weapons'];
    }

    if ($weapons == " ") {
      unset($weapons);
    }

    $shipinfo['hullid']    = $hull;
    $shipinfo['weapons']   = $weapons;
    $shipinfo['equipment'] = $equipment;

    if (!empty($hullid)) {
      $hull = $hullid;
    }

    $resc = dbaccess::query("SELECT * FROM hulls WHERE id='$hull'");
    while ($ra = dbaccess::getarray($resc)) {
      $shipinfo['hullname']    = $ra['name'];
      $shipinfo['cargoslots']  = $ra['cargoslots'];
      $shipinfo['shieldslots'] = $ra['shieldslots'];
      $shipinfo['armourslots'] = $ra['armourslots'];
      $shipinfo['engineslots'] = $ra['engineslots'];
      $shipinfo['equipslots']  = $ra['equipslots'];
      $shipinfo['primslots']   = $ra['primslots'];
      $shipinfo['secslots']    = $ra['secslots'];
      $shipinfo['tps']         = $ra['tps'];
      $shipinfo['startprop']   = $ra['startprop'];
    }

    if (!empty($equipment)) {
      $tequip = explode(",", $equipment);
      $key = array_search("bck", $tequip);

      if ($key !== FALSE) {
        $shipinfo['tps'] += 10;
      }
    }

    $props = explode(",", $properties);
    foreach ($props as $i => $value) {
      $cprops = explode(":", $value);
      $cshipprops[$cprops[0]] = $cprops[1];
    }

    $propshull = explode(",", $shipinfo['startprop']);
    foreach ($propshull as $i => $value) {
      $cpropshull = explode(":", $value);
      $cshippropshull[$cpropshull[0]] = $cpropshull[1];
    }

    $ship_tech['shields'] = $technology[1]['prop'];
    $ship_tech['armour']  = $technology[2]['prop'];
    $ship_tech['cargo']   = $technology[3]['prop'];
    $ship_tech['engines'] = $technology[4]['prop'];

    $max_shields_h = explode(":", $ship_tech['shields']);
    $max_shields = $max_shields_h[1];
    $max_shields *= $shipinfo['shieldslots'];

    $max_armour_h = explode(":", $ship_tech['armour']);
    $max_armour = $max_armour_h[1];
    $max_armour *= $shipinfo['armourslots'];

    $max_holds_h = explode(":", $ship_tech['cargo']);
    $max_holds = $max_holds_h[1];
    $max_holds *= $shipinfo['cargoslots'];

    $max_engines_h = explode(":", $ship_tech['engines']);
    $max_engines = $max_engines_h[1];
    $max_engines *= $shipinfo['engineslots'];

    $shipinfo['shipid']         = $shipid;
    $shipinfo['currentshields'] = $cshipprops['S'];
    $shipinfo['currentarmour']  = $cshipprops['A'];
    $shipinfo['currentholds']   = $cshipprops['C'];
    $shipinfo['currentengines'] = $cshipprops['E'];
    $shipinfo['hullshields']    = $cshippropshull['S'];
    $shipinfo['hullarmour']     = $cshippropshull['A'];
    $shipinfo['hullholds']      = $cshippropshull['C'];
    $shipinfo['hullengines']    = $cshippropshull['E'];
    $shipinfo['maxshields']     = $max_shields;
    $shipinfo['maxarmour']      = $max_armour;
    $shipinfo['maxholds']       = $max_holds;
    $shipinfo['maxengines']     = $max_engines;

    $shipinfo['currentcd']      = 0;
    $shipinfo['currentsd']      = 0;

    $currentdrones = 0;
    if (!empty($cshipprops['CD'])) {
      $currentdrones += $cshipprops['CD'];
      $shipinfo['currentcd'] = $cshipprops['CD'];
    }

    if (!empty($cshipprops['SD'])) {
      $currentdrones += $cshipprops['SD'];
      $shipinfo['currentsd'] = $cshipprops['SD'];
    }

    $shipinfo['currentdrones'] = $currentdrones;

    $lvl_tmp = 0;
    $lvl_tmp += ($shipinfo['currentshields'] / $max_shields_h[1]);
    $lvl_tmp += ($shipinfo['currentarmour'] / $max_armour_h[1]);
    $lvl_tmp += ($shipinfo['currentholds'] / $max_holds_h[1]);
    $lvl_tmp += ($shipinfo['currentengines'] / $max_engines_h[1]);

    if (empty($weapons)) {
      $weapons = array();
    } else {
      $weapons = explode(",", $weapons);
    }
    $lvl_tmp += array_sum($weapons);

    $shipinfo['level'] = ceil($lvl_tmp/15);

    return $shipinfo;
  }

  /**
   * Decrease a ships equipment when attacked
   *
   * This function will accept the type of equipment, the amount of damage and
   * the player id of the player being attacked, it will then decrease the
   * needed equipment by the needed amount.
   *
   * @param integer $equip
   * @param integer $amount
   * @param integer $playerid
   */
  function changeequipattack($equip, $amount, $playerid,$gameid) {
    $shipinfo = ship::getshipinfo($playerid,$gameid);
    $techinfo = tech::gettechinfo();

    $cshields = $shipinfo['currentshields'];
    $carmour  = $shipinfo['currentarmour'];
    $cholds   = $shipinfo['currentholds'];
    $cengines = $shipinfo['currentengines'];
    $csd      = $shipinfo['currentsd'];
    $ccd      = $shipinfo['currentcd'];

    switch ($equip) {
      case 'shields' :
        $cshields -= ($amount);
        if ($cshields <= 0) {
          $cshields = 0;
        }
        break;
      case 'armour' :
        $carmour -= ($amount);
        if ($carmour <= 0) {
          $carmour = 0;
        }
        break;
      case 'holds' :
        $cholds -= ($amount);
        if ($cholds <= 0) {
          $cholds = 0;
        }
        break;
      case 'engines' :
        $cengines -= ($amount);
        if ($cengines <= 0) {
          $cengines = 0;
        }
        break;
      case 'cd' :
        $ccd -= ($amount);
        if ($ccd <= 0) {
          $ccd = 0;
        }
        break;
      case 'sd' :
        $csd -= ($amount);
        if ($csd <= 0) {
          $csd = 0;
        }
        break;
    }
    $newprop = 'S:'.$cshields.',A:'.$carmour.',C:'.$cholds.',E:'.$cengines.',CD:'.$ccd.',SD:'.$csd;
    dbaccess::query("UPDATE `".$gameid."_ships` SET properties='".$newprop."' WHERE id='".$shipinfo['shipid']."'");

  }

  /**
   * Change equipment when purchased.
   *
   * This function will update the equipment when new equipment is purchased
   * from the Equipment Dealer.
   *
   * @param integer $equip
   * @param integer $amount
   * @param integer $playerid
   */
  function changeequip($equip, $amount, $playerid, $gameid) {
    $shipinfo = ship::getshipinfo($playerid,$gameid);
    $techinfo = tech::gettechinfo();

    $incby_tmp = explode(":", $techinfo[1]['prop']);
    $incby['shields'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[2]['prop']);
    $incby['armour'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[3]['prop']);
    $incby['holds'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[4]['prop']);
    $incby['engines'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[5]['prop']);
    $incby['cd'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[6]['prop']);
    $incby['sd'] = $incby_tmp[1];

    $incby_tmp = $techinfo[7]['prop'];
    $incby['bck'] = 'bck';

    $incby_tmp = $techinfo[8]['prop'];
    $incby['jmp'] = 'jmp';

    $cshields = $shipinfo['currentshields'];
    $carmour  = $shipinfo['currentarmour'];
    $cholds   = $shipinfo['currentholds'];
    $cengines = $shipinfo['currentengines'];
    $ccd      = $shipinfo['currentcd'];
    $csd      = $shipinfo['currentsd'];

    $maxshields = $shipinfo['maxshields'];
    $maxarmour  = $shipinfo['maxarmour'];
    $maxholds   = $shipinfo['maxholds'];
    $maxengines = $shipinfo['maxengines'];

    $stop = FALSE;
    $notequip = FALSE;
    switch ($equip) {
      case 'shields' :
        $cshields += ($incby['shields'] * $amount);
        if ($cshields > $maxshields) {
          $stop = TRUE;
        }
        break;
      case 'armour' :
        $carmour += ($incby['armour'] * $amount);
        if ($carmour > $maxarmour) {
          $stop = TRUE;
        }
        break;
      case 'holds' :
        $cholds += ($incby['holds'] * $amount);
        if ($cholds > $maxholds) {
          $stop = TRUE;
        }
        break;
      case 'engines' :
        $cengines += ($incby['engines'] * $amount);
        if ($cengines > $maxengines) {
          $stop = TRUE;
        }
        break;
      case 'cd' :
        $ccd += ($incby['cd'] * $amount);
        break;
      case 'sd' :
        $csd += ($incby['sd'] * $amount);
        break;
      case 'bck' :
        $notequip = TRUE;
        break;
      case 'jmp' :
        $notequip = TRUE;
        break;

    }

    if ($stop == FALSE) {
      if ($notequip == FALSE) {
        $newprop = 'S:'.$cshields.',A:'.$carmour.',C:'.$cholds.',E:'.$cengines.',CD:'.$ccd.',SD:'.$csd;
        dbaccess::query("UPDATE `".$gameid."_ships` SET properties='".$newprop."' WHERE id='".$shipinfo['shipid']."'");
      } else {
        $res = dbaccess::query("SELECT equipment FROM `".$gameid."_ships` WHERE id='".$shipinfo['shipid']."'");
        $res = dbaccess::getrow($res);
        $currequip = $res[0];

        if (empty($currequip)) {
          $nequip = $equip;
        } else {
          $nequip = explode(",", $currequip);
          $nequip[] = $equip;
          $nequip = implode(",", $nequip);
        }

        dbaccess::query("UPDATE `".$gameid."_ships` SET equipment='$nequip' WHERE id='".$shipinfo['shipid']."'");
      }
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Change cargo
   *
   * This function will change the goods that are currently in the holds of
   * the ship.
   *
   * @param integer $playerid
   * @param string $holds
   */
  function updateholds($playerid,$holds,$gameid) {
    dbaccess::query("UPDATE ".$gameid."_players SET cargo='$holds' WHERE id='$playerid'");
  }

  /**
   * Return free cargo space
   *
   * This function will return the free cargo space for the current ship.
   *
   * {@source}
   *
   * <b>Line 2 - 4</b>, get all the information needed about the current player.
   *
   * <b>Line 6</b>, basically when someone sells something and there is nothing
   * left in the cargo hold then the cargo field in the database will be empty.
   * So here we first check to see if there is nothing in the hold, if there is
   * something we then proceded to explode the values and then count the amount
   * of holds used up for the current cargo.
   * If there is no cargo then the free space is equal to the amount of cargo
   * holds the player has on his/her ship.
   *
   * @param integer $playerid
   * @return integer The free space in the holds
   */
  function getfreespace($playerid,$gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $cargo = $playerinf['cargo'];
    $holds = ship::getshipinfo($playerid,$gameid);

    if (!empty($cargo)) {
      $cargo = explode(",", $cargo);
      foreach ($cargo as $i => $value) {
        $thiscargoqty = explode(":", $value);
        $thecargoqty[$thiscargoqty[0]] = $thiscargoqty[1];
      }

      $totalcargo = array_sum($thecargoqty);
      $freespace  = $holds['currentholds'] - $totalcargo;
    } else {
      $shipinf = ship::getshipinfo($playerid,$gameid);
      $freespace = $shipinf['currentholds'];
    }

    return $freespace;
  }

  /**
   * Get the current cargo being carried
   *
   * This function will return the types of good and how much is being
   * carried at the moment this function has been called.
   *
   * @param integer $playerid
   * @return array Cargo
   */
  function getcurrentcargo($playerid,$gameid) {
    $playerinf = player::getplayerinfo($playerid,$gameid);
    $cargo = $playerinf['cargo'];

    if (!empty($cargo)) {
      $cargo = explode(",", $cargo);
      foreach ($cargo as $i => $value) {
        $thiscargoqty = explode(":", $value);
        $thecargoqty[$thiscargoqty[0]] = $thiscargoqty[1];
      }
      return $thecargoqty;
    } else {
      return array();
    }
  }

  /**
   * Return the name of a cargo
   *
   * This function will return the name of a cargo from the given good id.
   *
   * @param integer $cargoid
   * @return string Name of cargo
   */
  function getcargoname($cargoid) {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$cargoid'");
    while ($ra = dbaccess::getarray($resc)) {
      $name = $ra['name'];
    }

    return $name;
  }

}

?>
