<?php

  /**
   * Database Access File
   *
   * This file contains all the procedures that are needed for ingame functions
   * aswell as database access.
   *
   * @package TheMerchant
   * @author Shaun Steenkamp
   * @filesource
   */

// Checks to see whether a session has been started already
$sess_name = session_name();
if (empty($sess_name)) {
  session_start();
}
/**
 * DbAccess class
 *
 * This class contains the functions needed for database access.
 *
 * @todo Replace the DbAccess class with AdoDB code.
 */

class DbAccess {

  /**
   * DbAccess function
   *
   * This function will automatically be called when a new instance of the
   * DbAccess class is created, it will connect to the server and select
   * the required database.
   */
  function DbAccess() {
    global $database;
    mysql_connect($database['server'], $database['username'], $database['password']) or die('Could not connect: '.mysql_error());
    mysql_select_db($database['database']) or die('Could not select database: '.mysql_error());
  }

  /**
   * query function
   *
   * This function will run the passed query in the database.
   *
   * @param string $qry
   * @return string MySQL result resource
   */
  function query($qry) {
    $res = mysql_query($qry) or die('Error in query: '.mysql_error());
    return $res;
  }

  /**
   * numResults function
   *
   * This function will return the number of results found in the database
   * according to the result resource that has been passed to it
   *
   * @param mixed $res
   * @return integer The amount of records fount
   */
  function numResults($res) {
    $num = mysql_num_rows($res);
    return $num;
  }

  /**
   * getArray function
   *
   * This function performs the same function of the MySQL 'mysql_fetch_array'
   * function, it just has a shorter name
   *
   * @param mixed $res
   * @return array The result array
   */
  function getArray($res) {
    return mysql_fetch_array($res);
  }

  /**
   * getRow function
   *
   * This function will return the information from a row
   *
   * @param mixed $res
   * @return array The row info
   */
  function getRow($res) {
    return mysql_fetch_row($res);
  }

  /**
   * insertID function
   *
   * This function is the same as its MySQL counterpart
   *
   * @return integer The id of the last inserted record
   */
  function insertId() {
    return mysql_insert_id();
  }
}

/**
 * Game operations class<br>
 * Better documentation for this class' functions is being worked on.
 *
 * This class handles all the operations needed for the game to run
 * smoothly, it is becoming quite big so I am considering splitting
 * it up into smaller classes with more descriptive names, for instance,
 * use a class called Player for all the functions that handle the player.
 *
 *
 *
 * @todo Try to split the CurrentSector class into smaller more manageable classes.
 */
class CurrentSector {

  /**
   * Return sector id from record id
   *
   * This function will take the record id (from the database) and retrieve
   * the sector id (the id displayed to the players) to be returned.
   *
   * @param integer $id
   * @return integer The sector id of the given record id
   */
  function secid($id,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE id='$id'");
    while ($rc = dbaccess::getArray($resc)) {
      $secid = $rc['sector_id'];
    }

    return $secid;
  }

  /**
   * Return id from sector x and y coordinates
   *
   * This function will take the X and Y coordinates of a sector and return the
   * record id (not the one displayed in the game) for further usage.
   *
   * @param integer $x
   * @param integer $y
   * @return integer The record id of a sector
   */
  function getid($x,$y,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE xcoord = '$x' AND ycoord = '$y'");
    while ($rc = dbaccess::getArray($resc)) {
      $id = $rc['id'];
    }

    return $id;
  }

  /**
   * Return the record id from the sector id
   *
   * This function will accept the sector id of a sector (the id displayed in the game)
   * and return the record id of the sector (the id not shown in the game).
   *
   * @param integer $secid
   * @return integer Record id of sector
   */
  function idfromsecid($secid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE sector_id='$secid'");
    while ($rc = dbaccess::getArray($resc)) {
      $id = $rc['id'];
    }

    return $id;
  }

  /**
   * Return port id of a port in the current sector
   *
   * This function will accept the record id of a sector and return the port
   * id of the port located there. If there is no port present this function
   * will return 0 indicating no port.
   *
   * @param integer $id
   * @return integer ID of port
   */
  function hasport($id,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE id='$id'");
    while ($rc = dbaccess::getArray($resc)) {
      $port_id = $rc['port_id'];
    }

    return $port_id;
  }

  function hasstat($id,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_stations` WHERE secid='$id'");
    while ($rc = dbaccess::getArray($resc)) {
      $statid = $rc['id'];
    }

    return $statid;
  }

  /**
   * Checks to see if there is a station present
   *
   * This function will return 1 if there is a station present at the
   * current port, or 0 if there is no station.
   *
   * @param integer $id
   * @return integer 1 or 0
   */
  function hasStation($id,$gameid) {
    global $station_e;
    $station_e = 0;
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$id'");
    while ($rc = dbaccess::getarray($resc)) {
      $station_e = $rc['station'];
    }

    return $station_e;
  }

  /**
   * Return port name
   *
   * This function will return the name of the current port, if the port is
   * a station it will return a name, otherwise it will return the sector id
   * (the one displayed in the game).
   *
   * @param integer $id
   * @return mixed Name or Id of port
   */
  function getportname($id,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$id'");
    while ($rc = dbaccess::getArray($resc)) {
      $port_name = $rc['port_name'];
      $sector_id = $rc['sector_id'];
    }

    if (empty($port_name)) {
      return $sector_id;
    } else {
      return $port_name;
    }
  }

  /**
   * Get the selling price of a good
   *
   * This function will calculate the selling price of a good depending on how much
   * stock is at that location, hence the dynamic prices.
   *
   * @param integer $good
   * @param integer $qty
   * @return integer The selling price of a good
   */
  function getsellprice($good,$qty,$up = '0') {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$good'");
    while ($rc = dbaccess::getArray($resc)) {
      $goodvalue = $rc['value'];
    }

    $max_demand = 10000;

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $value = pow(($max_demand/$qty),0.8)*$goodvalue;

	settype($value,"integer");

	return $value;
  }

  /**
   * Get the buying price of a good
   *
   * This function will calculate the buying price of a good depending on how
   * much stock is available.
   *
   * @param integer $good
   * @param integer $qty
   * @return integer The buying price of a good
   */
  function getbuyprice($good,$qty) {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$good'");
    while ($rc = dbaccess::getArray($resc)) {
      $goodvalue = $rc['value'];
    }

    $max_demand = 10000;

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $value = sqrt((1/pow(($qty/$max_demand),0.5))*sqrt(($max_demand - $qty))*$goodvalue);

	settype($value,"integer");

	return $value;
  }

  /**
   * Get the experience gained from a purchase
   *
   * This function will calculate the experience that is gained from a purchase
   * from a port.
   *
   * @param integer $good
   * @param integer $qty
   * @param integer $amnt
   * @return integer The experience gained from the purchase
   */
  function getexpbuy($good,$qty,$amnt) {

    $max_demand = 10000;

    $sellp = $this->getsellprice($good,$qty);
    $experience = (pow($max_demand/$qty,0.7) * $amnt) / ($sellp/3.5);

	settype($experience,"integer");

	if ($experience <0)$experience=0;

	return $experience;
  }

  /**
   * Get the experience gained from a sale
   *
   * This function will calculate the experience that is gained from a sale to
   * a port.
   *
   * @param integer $good
   * @param integer $qty
   * @param integer $amnt
   * @return integer The experience gained from the sale
   */
  function getexpsell($good,$qty,$amnt,$up = '0') {
    $max_demand = 10000;

    $buyp = $this->getbuyprice($good,$qty);

    $qty = ($qty == 0) ? $qty = 1 : $qty;
    $experience = sqrt((pow(1/($qty/$max_demand),0.7) * $amnt) * ($buyp));

	settype($experience,"integer");

	if ($experience <0)$experience=0;

	return $experience;
  }

  /**
   * Get the players current location
   *
   * This function will return the record id of the sector that the player is
   * currently located at, it does that using the username of the player and then
   * getting his/her player id to finally get the player location.
   *
   * @param string $username
   * @return integer Record id of the current sector
   */
  function getplayerloc($username,$gameid) {
    $playerid = currentsector::getplayerid($username,$gameid);

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $sec = $ra['location'];
    }

    return $sec;
  }

  /**
   * Get the player id from the username
   *
   * This function will return the player id of the currently logged in username.
   *
   * @param string $username
   * @return integer The player id of the user
   */
  function getplayerid($username,$gameid) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    if (dbaccess::numresults($resc) > 0) {
      while ($ra = dbaccess::getarray($resc)) {
        $userid = $ra['id'];
      }
      $resa = dbaccess::query("SELECT id FROM `".$gameid."_players` WHERE userid='$userid'");
      $resa = dbaccess::getrow($resa);
      $playerid = $resa[0];
    } else {
      $playerid = 0;
    }

    return $playerid;
  }

  /**
   * Get the player id using the player name
   *
   * This function will accept the player name, and then return his/her player id
   *
   * @param string $playername
   * @return integer The player id of the player
   */
  function getplayeridfpn($playername,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE playername='$playername'");
    if (dbaccess::numresults($resc) > 0) {
      while ($ra = dbaccess::getarray($resc)) {
        $playerid = $ra['id'];
      }
    } else {
      $playerid = 0;
    }

    return $playerid;
  }

  /**
   * Get the user id of the username
   *
   * This function will return the user id of the passed username.
   *
   * @param string $username
   * @return integer The user id of the username
   */
  function getuserid($username) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    while ($ra = dbaccess::getarray($resc)) {
      $ids = $ra['id'];
    }

    return $ids;

  }

  /**
   * Get the time zone for a user
   *
   * This function will accept a username and then return the timezone
   *
   * @param string $username
   * @return integer The timezone for the user
   */
  function getTimeZone($username) {
    $resc = dbaccess::query("SELECT * FROM users WHERE username='$username'");
    while ($ra = dbaccess::getarray($resc)) {
      $tzone = $ra['toffset'];
    }

    return $tzone;
  }

  /**
   * Return the id of the port where the player is docked
   *
   * This function will accept the players id, and then return the port id of
   * where the player is currently docked, if the player is not docked at the
   * moment this function will return 0.
   *
   * @param integer $playerid
   * @return integer The id of the port
   */
  function hasdocked($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $dockedat = $ra['docked'];
    }

    return $dockedat;
  }

  function hasdockedstation($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $dockedat = $ra['atstation'];
    }

    return $dockedat;
  }

  /**
   * Dock a player
   *
   * This function will accept the player id, and port id of where the player
   * wishes to dock, and then it will dock the player there.
   *
   * @param integer $playerid
   * @param integer $portid
   */
  function dock($playerid, $portid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET docked='$portid' WHERE id='$playerid'");
  }

  function statdock($playerid, $statid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET atstation='$statid' WHERE id='$playerid'");
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);

    $res = dbaccess::query("SELECT ships FROM `".$gameid."_stations` WHERE id='$statid'");
    $res = dbaccess::getrow($res);
    $ships = $res[0];

    if (!empty($ships)) {
      $ships = explode(",", $ships);
      $ships[] = $playerinf['shipid'];
      $ships = implode(",", $ships);

      dbaccess::query("UPDATE `".$gameid."_stations` SET ships='$ships'");
    } else {
      $ships = $playerinf['shipid'];
      dbaccess::query("UPDATE `".$gameid."_stations` SET ships='$ships'");
    }

  }

  /**
   * Undock a player
   *
   * This function will accept the player id of a player and then set their current
   * docked port to 0, which then makes all the other functions return that the
   * player is not longer docked.
   *
   * @param integer $playerid
   */
  function undock($playerid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET docked='0' WHERE id='$playerid'");
  }

  function statundock($playerid,$gameid) {
    dbaccess::query("UPDATE `".$gameid."_players` SET atstation='0' WHERE id='$playerid'");
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);

    $res = dbaccess::query("SELECT ships FROM `".$gameid."_stations` WHERE id='$statid'");
    $res = dbaccess::getrow($res);
    $ships = $res[0];

    if (!empty($ships)) {
      $ships = explode(",", $ships);
      $key = array_search($playerinf['shipid'], $ships);
      unset($ships[$key]);

      $ships = implode(",", $ships);

      dbaccess::query("UPDATE `".$gameid."_stations` SET ships='$ships'");
    } else {
      $ships = 0;
      dbaccess::query("UPDATE `".$gameid."_stations` SET ships='$ships'");
    }
  }

  /**
   * Get all the information on a player
   *
   * This function will accept the player id of a player and then return everything
   * about that player, which is current credits, hull etc.
   *
   * @param integer $playerid
   * @return array All the information about a player
   */
  function getplayerinfo($playerid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($resc)) {
      $playerinfo['name']       = $ra['playername'];
      $playerinfo['auth']       = $ra['auth'];
      $playerinfo['ship']       = $ra['shipname'];
      $playerinfo['alliance']   = $ra['allianceid'];
      $playerinfo['credits']    = $ra['credits'];
      $playerinfo['turns']      = $ra['turns'];
      $playerinfo['newbie']     = $ra['newbie'];
      $playerinfo['cargo']      = $ra['cargo'];
      $playerinfo['shipid']     = $ra['shipid'];
      $playerinfo['exp']        = $ra['exp'];
      $playerinfo['docked']     = $ra['docked'];
      $playerinfo['userid']     = $ra['userid'];
      $playerinfo['banner']     = $ra['playerban'];
      $playerinfo['galid']      = $ra['galid'];
    }

    $old_exp = 0;
    $init_count = 5;
    $lvl_counter = 1;
    $gotlevel = FALSE;

    while (!$gotlevel) {
      if (($playerinfo['exp'] >= $old_exp) and ($playerinfo['exp'] <= $init_count)) {
        $gotlevel = TRUE;
      } else {
        $lvl_counter += 1;
        $old_exp = $init_count;
        $init_count = $init_count + (103 * (pow($lvl_counter,2.1)));
      }
    }

    $playerinfo['level'] = $lvl_counter - 1;

    $playerinfo['req_exp'] = $init_count;

    return $playerinfo;
  }

  /**
   * Return info on a weapon
   *
   * This function will accept the weapon id of a weapon, and then return
   * everything about that weapon in an array
   *
   * @param integer $weaponid
   * @return array Info on a weapon
   */
  function getweaponinfo($weaponid) {
    $resc = dbaccess::query("SELECT * FROM weapons WHERE id='$weaponid'");
    while ($ra = dbaccess::getarray($resc)) {
      $weaponinfo['name']       = $ra['name'];
      $weaponinfo['hulls']      = $ra['hulls'];
      $weaponinfo['damage']     = $ra['damage'];
      $weaponinfo['baseprice']  = $ra['baseprice'];
      $weaponinfo['multiplier'] = $ra['multiplier'];
    }

    return $weaponinfo;
  }

  /**
   * Return info on all technology
   *
   * This function accepts no parameters, it returns all the information on
   * all technology in the database.
   *
   * @return array Technology info on all technology.
   */
  function gettechinfo() {
    $resc = dbaccess::query("SELECT * FROM technology");
    while ($ra = dbaccess::getarray($resc)) {
      $tech[$ra['id']]['name'] = $ra['name'];
      $tech[$ra['id']]['prop'] = $ra['properties'];
      $tech[$ra['id']]['bp']   = $ra['baseprice'];
      $tech[$ra['id']]['mult'] = $ra['multiplier'];
    }
    return $tech;
  }

  /**
   * Return station id
   *
   * This function is exactly the same as hasStation.
   *
   * @see hasStation()
   * @todo Deprecate this function because it is a duplicate
   */
  function isstation($portid,$gameid) {
    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$portid'");
    while ($ra = dbaccess::getarray($resc)) {
      $station = $ra['station'];
    }

    if ($station == 1) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Return info on a ship
   *
   * This function will return everything about a ship, it also calculates a lot of
   * other information that is useful to the game. It accepts the player id, the
   * hull id parameter is optional, the Ship Dealer script uses it to get information
   * on the hulls that the dealer sells.
   *
   * @param integer $playerid
   * @param integer $hullid
   * @return array Info on the ship or hull
   */

  function getshipinfo($playerid,$gameid,$hullid = '') {
    $technology = currentsector::gettechinfo();
    $playerinfo = currentsector::getplayerinfo($playerid,$gameid);
    $shipid = $playerinfo['shipid'];

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ships` WHERE id='$shipid'");
    while ($ra = dbaccess::getarray($resc)) {
      $hull = $ra['hullid'];
      $properties = $ra['properties'];
      $equipment = $ra['equipment'];
      $weapons = $ra['weapons'];
    }

    if ($weapons == " ") {
      unset($weapons);
    }

    $shipinfo['hullid']    = $hull;
    $shipinfo['weapons']   = $weapons;
    $shipinfo['equipment'] = $equipment;

    if (!empty($hullid)) {
      $hull = $hullid;
    }

    $resc = dbaccess::query("SELECT * FROM hulls WHERE id='$hull'");
    while ($ra = dbaccess::getarray($resc)) {
      $shipinfo['hullname']    = $ra['name'];
      $shipinfo['cargoslots']  = $ra['cargoslots'];
      $shipinfo['shieldslots'] = $ra['shieldslots'];
      $shipinfo['armourslots'] = $ra['armourslots'];
      $shipinfo['engineslots'] = $ra['engineslots'];
      $shipinfo['equipslots']  = $ra['equipslots'];
      $shipinfo['primslots']   = $ra['primslots'];
      $shipinfo['secslots']    = $ra['secslots'];
      $shipinfo['tps']         = $ra['tps'];
      $shipinfo['startprop']   = $ra['startprop'];
    }

    if (!empty($equipment)) {
      $tequip = explode(",", $equipment);
      $key = array_search("bck", $tequip);

      if ($key !== FALSE) {
        $shipinfo['tps'] += 10;
      }
    }

    $props = explode(",", $properties);
    foreach ($props as $i => $value) {
      $cprops = explode(":", $value);
      $cshipprops[$cprops[0]] = $cprops[1];
    }

    $propshull = explode(",", $shipinfo['startprop']);
    foreach ($propshull as $i => $value) {
      $cpropshull = explode(":", $value);
      $cshippropshull[$cpropshull[0]] = $cpropshull[1];
    }

    $ship_tech['shields'] = $technology[1]['prop'];
    $ship_tech['armour']  = $technology[2]['prop'];
    $ship_tech['cargo']   = $technology[3]['prop'];
    $ship_tech['engines'] = $technology[4]['prop'];

    $max_shields_h = explode(":", $ship_tech['shields']);
    $max_shields = $max_shields_h[1];
    $max_shields *= $shipinfo['shieldslots'];

    $max_armour_h = explode(":", $ship_tech['armour']);
    $max_armour = $max_armour_h[1];
    $max_armour *= $shipinfo['armourslots'];

    $max_holds_h = explode(":", $ship_tech['cargo']);
    $max_holds = $max_holds_h[1];
    $max_holds *= $shipinfo['cargoslots'];

    $max_engines_h = explode(":", $ship_tech['engines']);
    $max_engines = $max_engines_h[1];
    $max_engines *= $shipinfo['engineslots'];

    $shipinfo['shipid']         = $shipid;
    $shipinfo['currentshields'] = $cshipprops['S'];
    $shipinfo['currentarmour']  = $cshipprops['A'];
    $shipinfo['currentholds']   = $cshipprops['C'];
    $shipinfo['currentengines'] = $cshipprops['E'];
    $shipinfo['hullshields']    = $cshippropshull['S'];
    $shipinfo['hullarmour']     = $cshippropshull['A'];
    $shipinfo['hullholds']      = $cshippropshull['C'];
    $shipinfo['hullengines']    = $cshippropshull['E'];
    $shipinfo['maxshields']     = $max_shields;
    $shipinfo['maxarmour']      = $max_armour;
    $shipinfo['maxholds']       = $max_holds;
    $shipinfo['maxengines']     = $max_engines;

    $shipinfo['currentcd']      = 0;
    $shipinfo['currentsd']      = 0;

    $currentdrones = 0;
    if (!empty($cshipprops['CD'])) {
      $currentdrones += $cshipprops['CD'];
      $shipinfo['currentcd'] = $cshipprops['CD'];
    }

    if (!empty($cshipprops['SD'])) {
      $currentdrones += $cshipprops['SD'];
      $shipinfo['currentsd'] = $cshipprops['SD'];
    }

    $shipinfo['currentdrones'] = $currentdrones;

    $lvl_tmp = 0;
    $lvl_tmp += ($shipinfo['currentshields'] / $max_shields_h[1]);
    $lvl_tmp += ($shipinfo['currentarmour'] / $max_armour_h[1]);
    $lvl_tmp += ($shipinfo['currentholds'] / $max_holds_h[1]);
    $lvl_tmp += ($shipinfo['currentengines'] / $max_engines_h[1]);

    if (empty($weapons)) {
      $weapons = array();
    } else {
      $weapons = explode(",", $weapons);
    }
    $lvl_tmp += array_sum($weapons);

    $shipinfo['level'] = ceil($lvl_tmp);

    return $shipinfo;
  }

  function getshipinfo_parked($shipid,$gameid,$hullid = '') {
    $technology = currentsector::gettechinfo();

    $resc = dbaccess::query("SELECT * FROM `".$gameid."_ships` WHERE id='$shipid'");
    while ($ra = dbaccess::getarray($resc)) {
      $hull = $ra['hullid'];
      $properties = $ra['properties'];
      $equipment = $ra['equipment'];
      $weapons = $ra['weapons'];
    }

    if ($weapons == " ") {
      unset($weapons);
    }

    $shipinfo['hullid']    = $hull;
    $shipinfo['weapons']   = $weapons;
    $shipinfo['equipment'] = $equipment;

    if (!empty($hullid)) {
      $hull = $hullid;
    }

    $resc = dbaccess::query("SELECT * FROM hulls WHERE id='$hull'");
    while ($ra = dbaccess::getarray($resc)) {
      $shipinfo['hullname']    = $ra['name'];
      $shipinfo['cargoslots']  = $ra['cargoslots'];
      $shipinfo['shieldslots'] = $ra['shieldslots'];
      $shipinfo['armourslots'] = $ra['armourslots'];
      $shipinfo['engineslots'] = $ra['engineslots'];
      $shipinfo['equipslots']  = $ra['equipslots'];
      $shipinfo['primslots']   = $ra['primslots'];
      $shipinfo['secslots']    = $ra['secslots'];
      $shipinfo['tps']         = $ra['tps'];
      $shipinfo['startprop']   = $ra['startprop'];
    }

    if (!empty($equipment)) {
      $tequip = explode(",", $equipment);
      $key = array_search("bck", $tequip);

      if ($key !== FALSE) {
        $shipinfo['tps'] += 10;
      }
    }

    $props = explode(",", $properties);
    foreach ($props as $i => $value) {
      $cprops = explode(":", $value);
      $cshipprops[$cprops[0]] = $cprops[1];
    }

    $propshull = explode(",", $shipinfo['startprop']);
    foreach ($propshull as $i => $value) {
      $cpropshull = explode(":", $value);
      $cshippropshull[$cpropshull[0]] = $cpropshull[1];
    }

    $ship_tech['shields'] = $technology[1]['prop'];
    $ship_tech['armour']  = $technology[2]['prop'];
    $ship_tech['cargo']   = $technology[3]['prop'];
    $ship_tech['engines'] = $technology[4]['prop'];

    $max_shields_h = explode(":", $ship_tech['shields']);
    $max_shields = $max_shields_h[1];
    $max_shields *= $shipinfo['shieldslots'];

    $max_armour_h = explode(":", $ship_tech['armour']);
    $max_armour = $max_armour_h[1];
    $max_armour *= $shipinfo['armourslots'];

    $max_holds_h = explode(":", $ship_tech['cargo']);
    $max_holds = $max_holds_h[1];
    $max_holds *= $shipinfo['cargoslots'];

    $max_engines_h = explode(":", $ship_tech['engines']);
    $max_engines = $max_engines_h[1];
    $max_engines *= $shipinfo['engineslots'];

    $shipinfo['shipid']         = $shipid;
    $shipinfo['currentshields'] = $cshipprops['S'];
    $shipinfo['currentarmour']  = $cshipprops['A'];
    $shipinfo['currentholds']   = $cshipprops['C'];
    $shipinfo['currentengines'] = $cshipprops['E'];
    $shipinfo['hullshields']    = $cshippropshull['S'];
    $shipinfo['hullarmour']     = $cshippropshull['A'];
    $shipinfo['hullholds']      = $cshippropshull['C'];
    $shipinfo['hullengines']    = $cshippropshull['E'];
    $shipinfo['maxshields']     = $max_shields;
    $shipinfo['maxarmour']      = $max_armour;
    $shipinfo['maxholds']       = $max_holds;
    $shipinfo['maxengines']     = $max_engines;

    $shipinfo['currentcd']      = 0;
    $shipinfo['currentsd']      = 0;

    $currentdrones = 0;
    if (!empty($cshipprops['CD'])) {
      $currentdrones += $cshipprops['CD'];
      $shipinfo['currentcd'] = $cshipprops['CD'];
    }

    if (!empty($cshipprops['SD'])) {
      $currentdrones += $cshipprops['SD'];
      $shipinfo['currentsd'] = $cshipprops['SD'];
    }

    $shipinfo['currentdrones'] = $currentdrones;

    $lvl_tmp = 0;
    $lvl_tmp += ($shipinfo['currentshields'] / $max_shields_h[1]);
    $lvl_tmp += ($shipinfo['currentarmour'] / $max_armour_h[1]);
    $lvl_tmp += ($shipinfo['currentholds'] / $max_holds_h[1]);
    $lvl_tmp += ($shipinfo['currentengines'] / $max_engines_h[1]);

    if (empty($weapons)) {
      $weapons = array();
    } else {
      $weapons = explode(",", $weapons);
    }
    $lvl_tmp += array_sum($weapons);

    $shipinfo['level'] = ceil($lvl_tmp);

    return $shipinfo;
  }

  /**
   * Decrease a ships equipment when attacked
   *
   * This function will accept the type of equipment, the amount of damage and
   * the player id of the player being attacked, it will then decrease the
   * needed equipment by the needed amount.
   *
   * @param integer $equip
   * @param integer $amount
   * @param integer $playerid
   */
  function changeequipattack($equip, $amount, $playerid,$gameid) {
    $shipinfo = currentsector::getshipinfo($playerid,$gameid);
    $techinfo = currentsector::gettechinfo();

    $cshields = $shipinfo['currentshields'];
    $carmour  = $shipinfo['currentarmour'];
    $cholds   = $shipinfo['currentholds'];
    $cengines = $shipinfo['currentengines'];
    $csd      = $shipinfo['currentsd'];
    $ccd      = $shipinfo['currentcd'];

    switch ($equip) {
      case 'shields' :
        $cshields -= ($amount);
        if ($cshields <= 0) {
          $cshields = 0;
        }
        break;
      case 'armour' :
        $carmour -= ($amount);
        if ($carmour <= 0) {
          $carmour = 0;
        }
        break;
      case 'holds' :
        $cholds -= ($amount);
        if ($cholds <= 0) {
          $cholds = 0;
        }
        break;
      case 'engines' :
        $cengines -= ($amount);
        if ($cengines <= 0) {
          $cengines = 0;
        }
        break;
      case 'cd' :
        $ccd -= ($amount);
        if ($ccd <= 0) {
          $ccd = 0;
        }
        break;
      case 'sd' :
        $csd -= ($amount);
        if ($csd <= 0) {
          $csd = 0;
        }
        break;
    }
    $newprop = 'S:'.$cshields.',A:'.$carmour.',C:'.$cholds.',E:'.$cengines.',CD:'.$ccd.',SD:'.$csd;
    dbaccess::query("UPDATE `".$gameid."_ships` SET properties='".$newprop."' WHERE id='".$shipinfo['shipid']."'");

  }

  /**
   * Change equipment when purchased.
   *
   * This function will update the equipment when new equipment is purchased
   * from the Equipment Dealer.
   *
   * @param integer $equip
   * @param integer $amount
   * @param integer $playerid
   */
  function changeequip($equip, $amount, $playerid, $gameid) {
    $shipinfo = currentsector::getshipinfo($playerid,$gameid);
    $techinfo = currentsector::gettechinfo();

    $incby_tmp = explode(":", $techinfo[1]['prop']);
    $incby['shields'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[2]['prop']);
    $incby['armour'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[3]['prop']);
    $incby['holds'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[4]['prop']);
    $incby['engines'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[5]['prop']);
    $incby['cd'] = $incby_tmp[1];

    $incby_tmp = explode(":", $techinfo[6]['prop']);
    $incby['sd'] = $incby_tmp[1];

    $incby_tmp = $techinfo[7]['prop'];
    $incby['bck'] = 'bck';

    $cshields = $shipinfo['currentshields'];
    $carmour  = $shipinfo['currentarmour'];
    $cholds   = $shipinfo['currentholds'];
    $cengines = $shipinfo['currentengines'];
    $ccd      = $shipinfo['currentcd'];
    $csd      = $shipinfo['currentsd'];

    $maxshields = $shipinfo['maxshields'];
    $maxarmour  = $shipinfo['maxarmour'];
    $maxholds   = $shipinfo['maxholds'];
    $maxengines = $shipinfo['maxengines'];

    $stop = FALSE;
    $notequip = FALSE;
    switch ($equip) {
      case 'shields' :
        $cshields += ($incby['shields'] * $amount);
        if ($cshields > $maxshields) {
          $stop = TRUE;
        }
        break;
      case 'armour' :
        $carmour += ($incby['armour'] * $amount);
        if ($carmour > $maxarmour) {
          $stop = TRUE;
        }
        break;
      case 'holds' :
        $cholds += ($incby['holds'] * $amount);
        if ($cholds > $maxholds) {
          $stop = TRUE;
        }
        break;
      case 'engines' :
        $cengines += ($incby['engines'] * $amount);
        if ($cengines > $maxengines) {
          $stop = TRUE;
        }
        break;
      case 'cd' :
        $ccd += ($incby['cd'] * $amount);
        break;
      case 'sd' :
        $csd += ($incby['sd'] * $amount);
        break;
      case 'bck' :
        $notequip = TRUE;
        break;

    }

    if ($stop == FALSE) {
      if ($notequip == FALSE) {
        $newprop = 'S:'.$cshields.',A:'.$carmour.',C:'.$cholds.',E:'.$cengines.',CD:'.$ccd.',SD:'.$csd;
        dbaccess::query("UPDATE `".$gameid."_ships` SET properties='".$newprop."' WHERE id='".$shipinfo['shipid']."'");
      } else {
        $res = dbaccess::query("SELECT equipment FROM `".$gameid."_ships` WHERE id='".$shipinfo['shipid']."'");
        $res = dbaccess::getrow($res);
        $currequip = $res[0];

        if (empty($currequip)) {
          $nequip = $equip;
        } else {
          $nequip = explode(",", $currequip);
          $nequip[] = $equip;
          $nequip = implode(",", $nequip);
        }

        dbaccess::query("UPDATE `".$gameid."_ships` SET equipment='$nequip' WHERE id='".$shipinfo['shipid']."'");
      }
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Change cargo
   *
   * This function will change the goods that are currently in the holds of
   * the ship.
   *
   * @param integer $playerid
   * @param string $holds
   */
  function updateholds($playerid,$holds,$gameid) {
    dbaccess::query("UPDATE ".$gameid."_players SET cargo='$holds' WHERE id='$playerid'");
  }

  /**
   * Decrease a players credits
   *
   * Accepts the players id and the amount of credits to decrease by.
   *
   * @param integer $playerid
   * @param integer $creds
   */
  function downcreds($playerid, $creds, $gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $currentcreds = $playerinf['credits'];
    $newcreds = $currentcreds - $creds;

    dbaccess::query("UPDATE `".$gameid."_players` SET credits = '$newcreds' WHERE id='$playerid'");
  }

  /**
   * Increase a players credits
   *
   * Accepts the players id and the amount of credits to increase by
   *
   * @param integer $playerid
   * @param integer $creds
   */
  function upcreds($playerid, $creds, $gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $currentcreds = $playerinf['credits'];
    $newcreds = $currentcreds + $creds;

    dbaccess::query("UPDATE `".$gameid."_players` SET credits = '$newcreds' WHERE id='$playerid'");
  }

  /**
   * Increase a players experience
   *
   * Accepts the players id and the amount of experience to increase by
   *
   * @param integer $playerid
   * @param integer $exp
   */
  function upexp($playerid, $exp,$gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $currentexp = $playerinf['exp'];
    $newexp = $currentexp + $exp;

    dbaccess::query("UPDATE `".$gameid."_players` SET exp = '$newexp' WHERE id='$playerid'");
  }

  /**
   * Decrease a players experience
   *
   * Accepts the players id and the amount of experience to decrease by
   *
   * @param integer $playerid
   * @param integer $exp
   */
  function downexp($playerid, $exp, $gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $currentexp = $playerinf['exp'];
    $newexp = $currentexp - $exp;

    dbaccess::query("UPDATE `".$gameid."_players` SET exp = '$newexp' WHERE id='$playerid'");
  }

  /**
   * Increase a players amount of turns
   *
   * This function will increase the amount of turns a player has, it is mainly
   * used by the event processor
   *
   * @param integer $playerid
   * @param integer $amnt
   */
  function upturns($playerid, $amnt, $gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $currentturns = $playerinf['turns'];
    $newturns = $currentturns + $amnt;
    dbaccess::query("UPDATE `".$gameid."_players` SET turns = '$newturns' WHERE id='$playerid'");
  }

  /**
   * Get all the info on a bank account
   *
   * This function will return all the relevant information about a bank account
   *
   * @param integer $playerid
   */
  function bankinfo($playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE playerid='$playerid'");
    while ($rac = dbaccess::getarray($res)) {
      $bankinfo['credits'] = $rac['credits'];
      $bankinfo['accid']   = $rac['id'];
    }

    if (empty($bankinfo)) {
      return FALSE;
    } else {
      return $bankinfo;
    }
  }

  /**
   * Increase credits in the bank
   *
   * This function will increase the amount of credits that are currently in the
   * bank account. The alliance variable is there incase the account that the
   * credits are being deposited to is an alliance account. This function will
   * also decrease the amount of credits in the players current credits.
   *
   * @param integer $playerid
   * @param integer $amnt
   * @param integer $alliance
   */
  function upbank($playerid, $amnt, $gameid, $alliance = '0', $anon = '0') {
    if ($anon != 0) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE anonid='$anon'");

      while ($r = dbaccess::getarray($res)) {
        $creds = $r['credits'];
        $accid = $r['id'];
      }

      $creds += $amnt;

      dbaccess::query("UPDATE `".$gameid."_bankaccounts` SET credits='$creds' WHERE id='$accid'");
    }

    if (($alliance == 0) and ($anon == 0)) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE playerid='$playerid'");
      if (dbaccess::numresults($res) == 0) {
        dbaccess::query("INSERT INTO `".$gameid."_bankaccounts` (playerid) VALUES ('$playerid')");
      }

      $bankinfo = currentsector::bankinfo($playerid,$gameid);
      $creds = $bankinfo['credits'];
      $accid = $bankinfo['accid'];

      $creds += $amnt;

      dbaccess::query("UPDATE `".$gameid."_bankaccounts` SET credits='$creds' WHERE id='$accid'");

    } elseif (($alliance != 0) and ($anon == 0)) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE allianceid='$alliance'");
      if (dbaccess::numresults($res) == 0) {
        return FALSE;
      }
    }

  }

  /**
   * Decrease credits in a bank account
   *
   * This function performs the exact opposite of the upbank function.
   *
   * @see upbank()
   * @param integer $playerid
   * @param integer $amnt
   * @param integer $alliance
   */
  function downbank($playerid, $amnt, $gameid, $alliance = '0', $anon = '0') {
    if ($anon != 0) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE anonid='$anon'");

      while ($r = dbaccess::getarray($res)) {
        $creds = $r['credits'];
        $accid = $r['id'];
      }

      $creds -= $amnt;

      dbaccess::query("UPDATE `".$gameid."_bankaccounts` SET credits='$creds' WHERE id='$accid'");
    }

    if (($alliance == 0) and ($anon == 0)) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE playerid='$playerid'");
      if (dbaccess::numresults($res) == 0) {
        dbaccess::query("INSERT INTO `".$gameid."_bankaccounts` (playerid) VALUES ('$playerid')");
      }

      $bankinfo = currentsector::bankinfo($playerid,$gameid);
      $creds = $bankinfo['credits'];
      $accid = $bankinfo['accid'];

      $creds -= $amnt;

      dbaccess::query("UPDATE `".$gameid."_bankaccounts` SET credits='$creds' WHERE id='$accid'");

    } elseif (($alliance != 0) and ($anon == 0)) {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_bankaccounts` WHERE allianceid='$alliance'");
      if (dbaccess::numresults($res) == 0) {
        return FALSE;
      }
    }

  }

  /**
   * Return free cargo space
   *
   * This function will return the free cargo space for the current ship.
   *
   * {@source}
   *
   * <b>Line 2 - 4</b>, get all the information needed about the current player.
   *
   * <b>Line 6</b>, basically when someone sells something and there is nothing
   * left in the cargo hold then the cargo field in the database will be empty.
   * So here we first check to see if there is nothing in the hold, if there is
   * something we then proceded to explode the values and then count the amount
   * of holds used up for the current cargo.
   * If there is no cargo then the free space is equal to the amount of cargo
   * holds the player has on his/her ship.
   *
   * @param integer $playerid
   * @return integer The free space in the holds
   */
  function getfreespace($playerid,$gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $cargo = $playerinf['cargo'];
    $holds = currentsector::getshipinfo($playerid,$gameid);

    if (!empty($cargo)) {
      $cargo = explode(",", $cargo);
      foreach ($cargo as $i => $value) {
        $thiscargoqty = explode(":", $value);
        $thecargoqty[$thiscargoqty[0]] = $thiscargoqty[1];
      }

      $totalcargo = array_sum($thecargoqty);
      $freespace  = $holds['currentholds'] - $totalcargo;
    } else {
      $shipinf = currentsector::getshipinfo($playerid,$gameid);
      $freespace = $shipinf['currentholds'];
    }

    return $freespace;
  }

  /**
   * Get the current cargo being carried
   *
   * This function will return the types of good and how much is being
   * carried at the moment this function has been called.
   *
   * @param integer $playerid
   * @return array Cargo
   */
  function getcurrentcargo($playerid,$gameid) {
    $playerinf = currentsector::getplayerinfo($playerid,$gameid);
    $cargo = $playerinf['cargo'];

    if (!empty($cargo)) {
      $cargo = explode(",", $cargo);
      foreach ($cargo as $i => $value) {
        $thiscargoqty = explode(":", $value);
        $thecargoqty[$thiscargoqty[0]] = $thiscargoqty[1];
      }
      return $thecargoqty;
    } else {
      return array();
    }
  }

  /**
   * Return the name of a cargo
   *
   * This function will return the name of a cargo from the given good id.
   *
   * @param integer $cargoid
   * @return string Name of cargo
   */
  function getcargoname($cargoid) {
    $resc = dbaccess::query("SELECT * FROM goods WHERE id='$cargoid'");
    while ($ra = dbaccess::getarray($resc)) {
      $name = $ra['name'];
    }

    return $name;
  }

  /**
   * Log a person in
   *
   * Log a person in
   *
   * @param string $username
   */
  function login($username) {
    dbaccess::query("UPDATE users SET loggedin='1' WHERE username='$username'");
  }

  /**
   * Log a person out
   *
   * Log a person out
   *
   * @param string $username
   */
  function logout($username) {
    dbaccess::query("UPDATE users SET loggedin='0' WHERE username='$username'");
  }

  /**
   * Check login validity
   *
   * This function will check the session variables according to what is in the
   * database, if something is wrong, it will return false and the player will
   * be logged out.
   *
   * {@source}
   *
   * @return boolean Valid or not.
   */
  function checklogin() {
    global $users;
    if (!empty($_SESSION['username'])) {
      $username = $_SESSION['username'];
      $password = $_SESSION['password'];

      $resc = dbaccess::query("SELECT * FROM users
                               WHERE username='$username'");

      while ($ra = dbaccess::getarray($resc)) {
        $user_db = $ra['username'];
        $pass_db = $ra['password'];
        $lastactive_db = $ra['lastactive'];
      }

      if ($password == $pass_db) {
        $lastactive = time();
        if (($lastactive - $users['logout_time']) >= $lastactive_db) {
          // Person was inactive for too long
          currentsector::logout($username);
          dbaccess::query("UPDATE users SET lastactive='$lastactive'
                           WHERE username='$username'");
          return FALSE;
        } else {
          dbaccess::query("UPDATE users SET lastactive='$lastactive'
                           WHERE username='$username'");
          return TRUE;
        }
      } else {
        return FALSE;
        currentsector::logout($username);
      }
    } else {
      return FALSE;
    }
  }

  /**
   * Return how many new messages a player has
   *
   * This function will check a players inbox and return how many messages are
   * new.
   *
   * @param integer $playerid
   * @return integer New mail
   */
  function getnewmail($playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_messages` WHERE `to` = '$playerid' AND `read` = '0'");
    $new = dbaccess::numresults($res);

    return $new;
  }

  /**
   * Return username of playerid
   *
   * This function will return the username of a player id.
   *
   * @param integer $playerid
   * @return string The username
   */
  function getusername($playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_players` WHERE id='$playerid'");
    while ($ra = dbaccess::getarray($res)) {
      $uid = $ra['userid'];
    }

    $res = dbaccess::query("SELECT * FROM users WHERE id='$uid'");
    while ($ra = dbaccess::getarray($res)) {
      $username = $ra['username'];
    }

    return $username;
  }

  function newupgoods($portid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE id='$portid'");
    while ($r = dbaccess::getarray($res)) {
      $sells = $r['sells'];
      $buys  = $r['buys'];
      $level = $r['level'];
    }

    unset($newupgoods);
    unset($newupreqs);
    $newupgoods = array();
    $newupreqs = array();

    $res = dbaccess::query("SELECT * FROM goods WHERE level='$level'");
    while ($r = dbaccess::getarray($res)) {
      $newupgoods[] = $r['id'];
      $newupreqs[] = $r['id'];
    }

    $sells = explode(",", $sells);
    foreach ($sells as $i => $value) {
      $arrkey = array_search($value, $newupgoods);
      unset($newupgoods[$arrkey]);
    }

    $buys = explode(",", $buys);
    foreach ($buys as $i => $value) {
      $arrkey = array_search($value, $newupgoods);
      unset($newupgoods[$arrkey]);
    }

    if (empty($newupgoods)) {
      $nlvl = $level + 1;
      dbaccess::query("UPDATE `".$gameid."_ports` SET level='$nlvl' WHERE id='$portid'");
      return FALSE;
    } else {
      $thenewgood = array_rand($newupgoods);
      unset($newupreqs[$thenewgood]);
      $thenewgood = $newupgoods[$thenewgood];

      foreach($sells as $i => $value) {
        $arrkey = array_search($value, $newupreqs);
        unset($newupreqs[$arrkey]);
      }

      $arrkey = array_search($thenewgood, $newupreqs);
      unset($newupreqs[$arrkey]);

      $newstring_ups = $thenewgood.":";

      $thenewreqs = array_rand($newupreqs,2);

      unset($fordbreqs);
      $fordbreqs = array();
      unset($fordbqty);
      $fordbqty = array();

      foreach ($thenewreqs as $i => $value) {
        $fordbreqs[] = $value;
        $fordbqty[] = $value."+30000";
      }

      $fordbreqs = implode("-", $fordbreqs);
      $fordbqty = implode("@", $fordbqty);

      $newstring_ups = $newstring_ups.$fordbreqs;
      $newstring_qty = $thenewgood.':'.$fordbqty;

      dbaccess::query("UPDATE `".$gameid."_ports` SET upgrade = '$newstring_ups',
                                        upgradeqty = '$newstring_qty'
                       WHERE id='$portid'");

      return TRUE;
    }
  }

  function hastopexit($x,$y, $gameid) {
    global $restop, $exists, $id, $rtop;
    $top = $y - 1;
    $restop = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE ycoord = '$top' AND xcoord = '$x'");

    $exists = dbaccess::numResults($restop);
    if ($exists != 0) {
      while ($rtop = dbaccess::getArray($restop)) {
        $id = $rtop['id'];
      }
    } else {
      $id = 0;
    }

    return $id;
  }

  function hasbottomexit($x,$y,$gameid) {
    global $restop, $exists, $id, $rtop;
    $bottom = $y + 1;
    $restop = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE ycoord = '$bottom' AND xcoord = '$x'");

    $exists = dbaccess::numResults($restop);
    if ($exists != 0) {
      while ($rtop = dbaccess::getArray($restop)) {
        $id = $rtop['id'];
      }
    } else {
      $id = 0;
    }

    return $id;
  }

  function hasleftexit($x,$y,$gameid) {
    global $restop, $exists, $id, $rtop;
    $left = $x - 1;
    $restop = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE ycoord = '$y' AND xcoord = '$left'");

    $exists = dbaccess::numResults($restop);
    if ($exists != 0) {
      while ($rtop = dbaccess::getArray($restop)) {
        $id = $rtop['id'];
      }
    } else {
      $id = 0;
    }

    return $id;
  }

  function hasrightexit($x,$y,$gameid) {
    global $restop, $exists, $id, $rtop;
    $right = $x + 1;
    $restop = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE ycoord = '$y' AND xcoord = '$right'");

    $exists = dbaccess::numResults($restop);
    if ($exists != 0) {
      while ($rtop = dbaccess::getArray($restop)) {
        $id = $rtop['id'];
      }
    } else {
      $id = 0;
    }

    return $id;
  }

  function pickupdrone($sector,$type,$amnt,$owner,$gameid) {
    if ($type == 'sd') {
      $res = dbaccess::query("SELECT * FROM drones WHERE secid='$sector' AND owner='$owner' AND type='sd'");
      $dexists = dbaccess::numresults($res);

      if ($dexists > 0) {
        $contains = dbaccess::getrow(dbaccess::query("SELECT id,amnt FROM drones WHERE secid='$sector' AND owner='$owner' AND type='sd'"));
        $rowid = $contains[0];
        $contains = $contains[1];

        $newsd = $contains - $amnt;

        if ($newsd != 0) {
          dbaccess::query("UPDATE drones SET amnt='$newsd' WHERE id='$rowid' LIMIT 1");
        } else {
          dbaccess::query("DELETE FROM drones WHERE id='$rowid'");
        }
      }

      $shipinfo = currentsector::getshipinfo($owner,$gameid);
      $current_sd = $shipinfo['currentsd'];
      $current_sd += $amnt;

      currentsector::changeequip('sd', $amnt, $owner);
    }

    if ($type == 'cd') {
      $res = dbaccess::query("SELECT * FROM drones WHERE secid='$sector' AND owner='$owner' AND type='cd'");
      $dexists = dbaccess::numresults($res);

      if ($dexists > 0) {
        $contains = dbaccess::getrow(dbaccess::query("SELECT id,amnt FROM drones WHERE secid='$sector' AND owner='$owner' AND type='cd'"));
        $rowid = $contains[0];
        $contains = $contains[1];

        $newsd = $contains - $amnt;

        if ($newsd != 0) {
          dbaccess::query("UPDATE drones SET amnt='$newsd' WHERE id='$rowid' LIMIT 1");
        } else {
          dbaccess::query("DELETE FROM drones WHERE id='$rowid'");
        }
      }

      $shipinfo = currentsector::getshipinfo($owner,$gameid);
      $current_sd = $shipinfo['currentcd'];
      $current_sd += $amnt;

      currentsector::changeequip('cd', $amnt, $owner);
    }
  }

  function dropdrone($sector,$type,$amnt,$owner,$gameid) {
    if ($type == 'sd') {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_drones` WHERE secid='$sector' AND owner='$owner' AND type='sd'");
      $dexists = dbaccess::numresults($res);

      if ($dexists > 0) {
        $contains = dbaccess::getrow(dbaccess::query("SELECT id,amnt FROM `".$gameid."_drones` WHERE secid='$sector' AND owner='$owner' AND type='sd'"));
        $rowid = $contains[0];
        $contains = $contains[1];

        $newsd = $contains + $amnt;

        dbaccess::query("UPDATE `".$gameid."_drones` SET amnt='$newsd' WHERE id='$rowid' LIMIT 1");
      } else {
        dbaccess::query("INSERT INTO `".$gameid."_drones` (secid,owner,type,stance,amnt,claim) VALUES (
                            '$sector',
                            '$owner',
                            '$type',
                            'neutral',
                            '$amnt',
                            '0')");
      }

      $shipinfo = currentsector::getshipinfo($owner,$gameid);
      $current_sd = $shipinfo['currentsd'];
      $current_sd -= $amnt;

      // While this functions name refers to being attacked, it only removes some equipment
      // from a players ship.
      currentsector::changeequipattack('sd', $amnt, $owner, $gameid);
    }

    if ($type == 'cd') {
      $res = dbaccess::query("SELECT * FROM `".$gameid."_drones` WHERE secid='$sector' AND owner='$owner' AND type='cd'");
      $dexists = dbaccess::numresults($res);

      if ($dexists > 0) {
        $contains = dbaccess::getrow(dbaccess::query("SELECT id,amnt FROM `".$gameid."_drones` WHERE secid='$sector' AND owner='$owner' AND type='cd'"));
        $rowid = $contains[0];
        $contains = $contains[1];

        $newsd = $contains + $amnt;

        dbaccess::query("UPDATE `".$gameid."_drones` SET amnt='$newsd' WHERE id='$rowid' LIMIT 1");
      } else {
        dbaccess::query("INSERT INTO `".$gameid."_drones` (secid,owner,type,stance,amnt,claim) VALUES (
                            '$sector',
                            '$owner',
                            '$type',
                            'neutral',
                            '$amnt',
                            '0')");
      }

      $shipinfo = currentsector::getshipinfo($owner,$gameid);
      $current_sd = $shipinfo['currentcd'];
      $current_sd -= $amnt;

      // While this functions name refers to being attacked, it only removes some equipment
      // from a players ship.
      currentsector::changeequipattack('cd', $amnt, $owner, $gameid);
    }
  }

  function hasdrones($sec,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_drones` WHERE secid='$sec'");
    $hasds = dbaccess::numresults($res);

    if ($hasds > 0) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  function playerhasdrones($sec,$playerid,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_drones` WHERE secid='$sec' AND owner='$playerid'");
    $hasds = dbaccess::numresults($res);

    if ($hasds > 0) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  function neareststation($sec,$gameid) {

    $res = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE id='$sec'");
    while ($r = dbaccess::getArray($res)) {
      $cx = $r['xcoord'];
      $cy = $r['ycoord'];
      $sector_id = $r['sector_id'];
    }

    $res = dbaccess::query("SELECT * FROM `".$gameid."_ports` WHERE station = '1'");
    $nx = $cx;
    $ny = $cy;
    $cdist = 0;
    $init = 0;
    while ($rac = dbaccess::getarray($res)) {
      $secid = $rac['sector_id'];
      $res2 = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE sector_id = '$secid'");
      while ($rec = dbaccess::getarray($res2)) {
        $tx = $rec['xcoord'];
        $ty = $rec['ycoord'];
      }
      $dist = abs($nx - $tx) + abs($ny - $ty);
      if ($dist == 0) {
        $cdist = 0;
        $dsecid = $secid;
      } elseif (($cdist == 0) and ($init == 0)) {
        $cdist = $dist;
        $dsecid = $secid;
        $init = 1;
      } elseif (($dist > 0) and ($dist < $cdist)) {
        $cdist = $dist;
        $dsecid = $secid;
      }
    }

    $returnvals['cdist']  = $cdist;
    $returnvals['dsecid'] = $dsecid;

    return $returnvals;

  }

  function getdist($sec,$sec2,$gameid) {

    $res = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE id='$sec'");
    while ($r = dbaccess::getArray($res)) {
      $cx = $r['xcoord'];
      $cy = $r['ycoord'];
      $sector_id = $r['sector_id'];
    }

    $nx = $cx;
    $ny = $cy;
    $cdist = 0;
    $init = 0;

    $res2 = dbaccess::query("SELECT * FROM `".$gameid."_sectors` WHERE id = '$sec2'");
    while ($rec = dbaccess::getarray($res2)) {
      $tx = $rec['xcoord'];
      $ty = $rec['ycoord'];
    }
    $dist = abs($nx - $tx) + abs($ny - $ty);
    if ($dist == 0) {
      $cdist = 0;
    } elseif (($cdist == 0) and ($init == 0)) {
      $cdist = $dist;
      $init = 1;
    } elseif (($dist > 0) and ($dist < $cdist)) {
      $cdist = $dist;
    }

    return $cdist;

  }
  
  function haswormhole($sec,$gameid) {
    $res = dbaccess::query("SELECT * FROM `".$gameid."_wormholes` WHERE secid=".$sec.";");
    if (dbaccess::numresults($res) > 0) {
      while ($r = dbaccess::getarray($res)) {
        $return['secid']     = $r['secid'];
        $return['secgoto']   = $r['secgoto'];
        $return['galid']     = $r['galid'];
        $return['gotogalid'] = $r['gotogalid'];
        $return['stability'] = $r['stability'];
	    $return['goto']	     = $r['goto'];
	  }
	} else {
	  $return = 0;
	}
	
	return $return;
  }

  function getgalname($secid,$gameid) {

    $res = dbaccess::query("SELECT `galid` FROM `".$gameid."_sectors` WHERE `id`='$secid' LIMIT 1");
    $res = dbaccess::getrow($res);
    $galid = $res[0];

    $res = dbaccess::query("SELECT `name` FROM `".$gameid."_galaxies` WHERE `id`='$galid' LIMIT 1");
    $res = dbaccess::getrow($res);
    return $res[0];

  }

}


?>
